use crate::utils;

pub fn execute_task01(content: &str) {
    let vector = utils::convert_to_string_slice(&content);
    let pairs: Vec<((i32, i32), (i32, i32))> = vector.iter().map(|line| convert_to_tuple(line)).collect();

    let fully_pairs: usize = pairs
        .into_iter()
        .filter(|pair| is_fully_in_other(pair.0, pair.1))
        .count();

    println!("Day04 - Task01 - Sum of fully included pairs: {}", fully_pairs);
}

pub fn execute_task02(content: &str) {
    let vector = utils::convert_to_string_slice(&content);
    let pairs: Vec<((i32, i32), (i32, i32))> = vector.iter().map(|line| convert_to_tuple(line)).collect();

    let fully_pairs: usize = pairs
        .into_iter()
        .filter(|pair| is_partial_in_other(pair.0, pair.1))
        .count();

    println!("Day04 - Task02 - Sum of partial included pairs: {}", fully_pairs);
}


fn convert_to_tuple(line: &str) -> ((i32, i32), (i32, i32)) {
   let pairs: Vec<(i32, i32)> = line
       .split(",")
       .map(|pair|  {
           let digits: Vec<i32> = pair.split("-").map(|digit| digit.parse::<i32>().unwrap()).collect();
           return (digits[0], digits[1]);
       })
       .collect();

    return (pairs[0], pairs[1]);
}

fn is_fully_in_other(first: (i32, i32), second: (i32,i32)) -> bool {
    if first.0 <= second.0 && first.1 >= second.1 { // second is fully in first
        return true;
    }

    if second.0 <= first.0 && second.1 >= first.1 { // first is fully in second
        return true;
    }

    return false
}

fn is_partial_in_other(first: (i32, i32), second: (i32, i32)) -> bool {
    if first.0 <= second.0 && first.1 >= second.0 { // 5-7, 6-9
        return true
    }

    if first.0 >= second.0 && first.0 <= second.1 { // 6-9, 5-7
        return true
    }

    return false
}

